@@ -1,33 +1,30 @@
 from django.core.exceptions import ValidationError
-from django.contrib.contenttypes.models import ContentType
 from rest_framework import generics, permissions, status
-from rest_framework.generics import get_object_or_404
 from rest_framework.response import Response
 from rest_framework.views import APIView
 
 from comment.validators import ValidatorMixin, ContentTypeValidator
 from comment.api.serializers import CommentSerializer, CommentCreateSerializer
 from comment.api.permissions import (
-    IsOwnerOrReadOnly, FlagEnabledPermission, CanChangeFlaggedCommentState
+    IsOwnerOrReadOnly, FlagEnabledPermission, CanChangeFlaggedCommentState, SubscriptionEnabled,
+    CanGetSubscribers, CanCreatePermission, UserPermittedOrReadOnly, CanBlockUsers
 )
-from comment.models import Comment, Reaction, ReactionInstance, Flag, FlagInstance
+from comment.models import Comment, Reaction, ReactionInstance, Flag, FlagInstance, Follower
 from comment.utils import get_comment_from_key, CommentFailReason
 from comment.messages import FlagError, EmailError
+from comment.views import BaseToggleFollowView, CommentCreateMixin, BaseToggleBlockingView
 
 
 class CommentCreate(ValidatorMixin, generics.CreateAPIView):
     serializer_class = CommentCreateSerializer
-    permission_classes = ()
+    permission_classes = (CanCreatePermission, UserPermittedOrReadOnly)
     api = True
 
     def get_serializer_context(self):
         self.validate(self.request)
         context = super().get_serializer_context()
-        context['user'] = self.request.user
-        context['model_name'] = self.model_name
-        context['app_name'] = self.app_name
-        context['model_id'] = self.model_id
-        context['parent_id'] = self.parent_id
+        context['model_obj'] = self.model_obj
+        context['parent_comment'] = self.parent_comment
         context['email'] = self.request.GET.get('email', None)
         return context
 
@@ -39,24 +36,19 @@ class CommentList(ContentTypeValidator, generics.ListAPIView):
 
     def get_queryset(self):
         self.validate(self.request)
-        model_name = self.model_name
-        pk = self.model_id
-        content_type_model = ContentType.objects.get(model=model_name.lower())
-        model_class = content_type_model.model_class()
-        model_obj = model_class.objects.filter(id=pk).first()
-        return Comment.objects.filter_parents_by_object(model_obj)
+        return Comment.objects.filter_parents_by_object(self.model_obj).select_related('user', 'reaction', 'flag')
 
 
 class CommentDetail(generics.RetrieveUpdateDestroyAPIView):
-    queryset = Comment.objects.all()
+    queryset = Comment.objects.all().select_related('user', 'reaction', 'flag')
     serializer_class = CommentSerializer
-    permission_classes = (permissions.IsAuthenticatedOrReadOnly, IsOwnerOrReadOnly)
+    permission_classes = (permissions.IsAuthenticatedOrReadOnly, IsOwnerOrReadOnly, UserPermittedOrReadOnly)
 
 
-class CommentDetailForReaction(generics.RetrieveAPIView):
-    queryset = Comment.objects.all()
+class CommentDetailForReaction(generics.UpdateAPIView):
+    queryset = Comment.objects.all().select_related('user', 'reaction', 'flag')
     serializer_class = CommentSerializer
-    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
+    permission_classes = (permissions.IsAuthenticatedOrReadOnly, UserPermittedOrReadOnly)
 
     def get_serializer_context(self):
         context = super().get_serializer_context()
@@ -64,7 +56,7 @@ class CommentDetailForReaction(generics.RetrieveAPIView):
         return context
 
     def post(self, request, *args, **kwargs):
-        comment = get_object_or_404(Comment, id=kwargs.get('pk'))
+        comment = self.get_object()
         reaction_type = kwargs.get('reaction', None)
         reaction_obj = Reaction.objects.get_reaction_object(comment)
         try:
@@ -81,10 +73,10 @@ class CommentDetailForReaction(generics.RetrieveAPIView):
         return Response(serializer.data, status=status.HTTP_200_OK)
 
 
-class CommentDetailForFlag(generics.RetrieveAPIView):
-    queryset = Comment.objects.all()
+class CommentDetailForFlag(generics.UpdateAPIView):
+    queryset = Comment.objects.all().select_related('user', 'reaction', 'flag')
     serializer_class = CommentSerializer
-    permission_classes = (permissions.IsAuthenticatedOrReadOnly, FlagEnabledPermission)
+    permission_classes = (permissions.IsAuthenticatedOrReadOnly, FlagEnabledPermission, UserPermittedOrReadOnly)
 
     def get_serializer_context(self):
         context = super().get_serializer_context()
@@ -92,7 +84,7 @@ class CommentDetailForFlag(generics.RetrieveAPIView):
         return context
 
     def post(self, request, *args, **kwargs):
-        comment = get_object_or_404(Comment, id=kwargs.get('pk'))
+        comment = self.get_object()
         flag = Flag.objects.get_for_comment(comment)
         reason = request.data.get('reason') or request.POST.get('reason')
         info = request.data.get('info') or request.POST.get('info')
@@ -105,10 +97,10 @@ class CommentDetailForFlag(generics.RetrieveAPIView):
         return Response(serializer.data, status=status.HTTP_200_OK)
 
 
-class CommentDetailForFlagStateChange(generics.RetrieveAPIView):
-    queryset = Comment.objects.all()
+class CommentDetailForFlagStateChange(generics.UpdateAPIView):
+    queryset = Comment.objects.all().select_related('user', 'reaction', 'flag')
     serializer_class = CommentSerializer
-    permission_classes = (CanChangeFlaggedCommentState, )
+    permission_classes = (CanChangeFlaggedCommentState, UserPermittedOrReadOnly)
 
     def get_serializer_context(self):
         context = super().get_serializer_context()
@@ -116,13 +108,8 @@ class CommentDetailForFlagStateChange(generics.RetrieveAPIView):
         return context
 
     def post(self, request, *args, **kwargs):
-        comment = get_object_or_404(Comment, id=kwargs.get('pk'))
+        comment = self.get_object()
         flag = Flag.objects.get_for_comment(comment)
-        if not comment.is_flagged:
-            return Response(
-                {'detail': FlagError.REJECT_UNFLAGGED_COMMENT},
-                status=status.HTTP_400_BAD_REQUEST
-            )
         state = request.data.get('state') or request.POST.get('state')
         try:
             state = flag.get_clean_state(state)
@@ -139,16 +126,46 @@ class CommentDetailForFlagStateChange(generics.RetrieveAPIView):
         return Response(serializer.data, status=status.HTTP_200_OK)
 
 
-class ConfirmComment(APIView):
-    @staticmethod
-    def get(request, *args, **kwargs):
+class ConfirmComment(APIView, CommentCreateMixin):
+    def get(self, request, *args, **kwargs):
         key = kwargs.get('key', None)
-        comment = get_comment_from_key(key)
+        temp_comment = get_comment_from_key(key)
 
-        if comment.why_invalid == CommentFailReason.BAD:
+        if temp_comment.why_invalid == CommentFailReason.BAD:
             return Response({'detail': EmailError.BROKEN_VERIFICATION_LINK}, status=status.HTTP_400_BAD_REQUEST)
 
-        if comment.why_invalid == CommentFailReason.EXISTS:
+        if temp_comment.why_invalid == CommentFailReason.EXISTS:
             return Response({'detail': EmailError.USED_VERIFICATION_LINK}, status=status.HTTP_200_OK)
 
-        return Response(CommentSerializer(comment.obj).data, status=status.HTTP_201_CREATED)
+        comment = self.perform_save(temp_comment.obj, request)
+
+        return Response(CommentSerializer(comment).data, status=status.HTTP_201_CREATED)
+
+
+class ToggleFollowAPI(BaseToggleFollowView, APIView):
+    api = True
+    response_class = Response
+    permission_classes = (SubscriptionEnabled, permissions.IsAuthenticated, UserPermittedOrReadOnly)
+
+    def post(self, request, *args, **kwargs):
+        self.validate(request)
+        return super().post(request, *args, **kwargs)
+
+
+class SubscribersAPI(ContentTypeValidator, APIView):
+    api = True
+    permission_classes = (CanGetSubscribers,)
+
+    def get(self, request, *args, **kwargs):
+        self.validate(request)
+        return Response({
+            'app_name': self.model_obj._meta.app_label,
+            'model_name': self.model_obj.__class__.__name__,
+            'model_id': self.model_obj.id,
+            'followers': Follower.objects.get_emails_for_model_object(self.model_obj)
+        })
+
+
+class ToggleBlockingAPI(BaseToggleBlockingView, APIView):
+    permission_classes = (CanBlockUsers,)
+    response_class = Response
