@@ -8,6 +8,8 @@ document.addEventListener('DOMContentLoaded', () => {
     let deleteModal = document.getElementById("Modal");
     let flagModal = document.getElementById('flagModal');
     let followModal = document.getElementById('followModal');
+    let blockModal = document.getElementById('blockModal');
+    let createAnonymousCommentModal = document.getElementById('createAnonymousCommentModal');
     let headers = {
         'X-Requested-With': 'XMLHttpRequest',
         'X-CSRFToken': csrfToken,
@@ -29,6 +31,7 @@ document.addEventListener('DOMContentLoaded', () => {
     };
 
     let showModal = modalElement => {
+        if (!modalElement) return;
         modalElement.style.display = 'block';
         setTimeout(() => {
             modalElement.classList.add('show-modal');
@@ -36,10 +39,21 @@ document.addEventListener('DOMContentLoaded', () => {
     };
 
     let hideModal = modalElement => {
+        if (!modalElement) return;
         modalElement.classList.remove('show-modal');
         modalElement.style.display = 'none';
     };
 
+    let hideCreateAnonymousCommentModal = () => {
+        if (!createAnonymousCommentModal) return;
+        let form = createAnonymousCommentModal.querySelector('.modal-body').querySelector('form');
+        if (form) {
+            createAnonymousCommentModal.querySelector('.modal-body').removeChild(form);
+        }
+        resetCreateForms();
+        hideModal(createAnonymousCommentModal);
+    };
+
     // show and hide child comments
     let replyLink = replyLinkElement => {
         getNthParent(replyLinkElement, 4).nextElementSibling.classList.toggle('d-none');
@@ -110,6 +124,48 @@ document.addEventListener('DOMContentLoaded', () => {
         }).join('&');
     };
 
+    let loadAnonymousModalForm = _form => {
+        let form = _form.cloneNode(true);
+        let textareaElement = form.querySelector('textarea');
+        textareaElement.hidden = true;
+        form.querySelector('button').setAttribute('data-email-required', 'false');
+        let emailElementData = `
+                    <div class="row">
+                        <div class="col-2">
+                            <label for="email">Email:</label>
+                        </div>
+                        <div class="col-10">
+                            <input id="DABEmail" class="form-control mr-2 w-100" type="email" name="email" required>
+                            <div class="error text-danger small mt-1"></div>
+                        </div>
+                    </div>`;
+        let emailElement = stringToDom(emailElementData, '.row');
+        let inputPlaceholder = form.querySelector('#inputPlaceholder');
+        inputPlaceholder.classList.remove('col-sm-9');
+        inputPlaceholder.classList.remove('col-md-10');
+        inputPlaceholder.classList.add('col-sm-8');
+        inputPlaceholder.appendChild(emailElement);
+        let buttonPlaceholder = form.querySelector('#buttonPlaceholder');
+        buttonPlaceholder.classList.remove('col-sm-3');
+        buttonPlaceholder.classList.remove('col-md-2');
+        buttonPlaceholder.classList.add('col-sm-4');
+        createAnonymousCommentModal.querySelector('.modal-body').appendChild(form);
+        showModal(createAnonymousCommentModal);
+        form.querySelector('#DABEmail').focus();
+    };
+
+    let resetCreateForms = () => {
+        let buttons = document.getElementsByClassName('js-comment-btn');
+        Array.prototype.forEach.call(buttons, element => {
+            element.setAttribute('disabled', 'disabled');
+        });
+        let inputs = document.getElementsByClassName("js-comment-input");
+        Array.prototype.forEach.call(inputs, element => {
+            element.setAttribute("style", "height: 31px;");
+            element.value = '';
+        });
+    };
+
     // create new comment
     let submitCommentCreateForm = form => {
         let errorElement = form.querySelector('.error');
@@ -121,6 +177,11 @@ document.addEventListener('DOMContentLoaded', () => {
         formData.page = urlParams.get('page');
         // this step is needed to append the form data to request.POST
         let formDataQuery = convertFormDataToURLQuery(formData);
+        let emailRequired = formButton.getAttribute('data-email-required') === 'true';
+        if (emailRequired) {
+            loadAnonymousModalForm(form);
+            return;
+        }
         fetch(url, {
             method: 'POST',
             headers: headers,
@@ -128,28 +189,30 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
-            if (result.error) {
-                // alert(result.error);
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            } else if (result.error) {
                 form.querySelector('.error').innerHTML = result.error;
                 form.querySelector('.error').classList.remove('d-none');
                 return;
             }
+            // reset form
+            resetCreateForms();
+            if (result.anonymous) {
+                createInfoElement(document.querySelector('.js-comment'), 'success', result.msg, 3);
+                form.reset();
+                hideModal(createAnonymousCommentModal);
+                createAnonymousCommentModal.querySelector('.modal-body').removeChild(form);
+                return;
+            }
+
             // parent comment
             if (formButton.getAttribute('value') === 'parent') {
                 // reload all comments only when posting parent comment
-                if (result.anonymous) {
-                    createInfoElement(form.closest('.js-comment'), 'success', result.msg, 3);
-                    form.reset();
-                    return;
-                }
                 document.getElementById("comments").outerHTML = result.data;
             } else {
                 // child comment
-                if (result.anonymous) {
-                    createInfoElement(form.closest('.js-parent-comment'), 'success', result.msg, 3);
-                    form.reset();
-                    return;
-                }
                 let childComment = stringToDom(result.data, '.js-child-comment');
                 form.parentNode.insertBefore(childComment, form);
                 // update number of replies
@@ -165,25 +228,20 @@ document.addEventListener('DOMContentLoaded', () => {
                 commentCount(1);
                 // update followBtn
                 let followButton = form.parentElement.previousElementSibling.querySelector(".js-comment-follow");
-                if (followButton){
+                if (followButton) {
                     followButton.querySelector('.comment-follow-icon').classList.add('user-has-followed');
                     followButton.querySelector('span').setAttribute('title', 'Unfollow this thread');
                 }
             }
-            formButton.setAttribute('disabled', 'disabled');
-            let elements = document.getElementsByClassName("js-comment-input");
-            Array.prototype.forEach.call(elements, element => {
-                element.setAttribute("style", "height: 31px;");
-                element.value = '';
-            });
             // remove pagination query when posting a new comment
             let uri = window.location.toString();
             if (uri.indexOf("?") > 0) {
                 let clean_uri = uri.substring(0, uri.indexOf("?"));
                 window.history.replaceState({}, document.title, clean_uri);
             }
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Unable to post your comment!, please try again"));
+            console.error(error);
         });
     };
 
@@ -194,6 +252,10 @@ document.addEventListener('DOMContentLoaded', () => {
         fetch(url, {headers: headers}).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             let editModeElement = stringToDom(result.data, '.js-comment-update-mode');
             commentContent.replaceWith(editModeElement);
             // set the focus on the end of text
@@ -203,8 +265,9 @@ document.addEventListener('DOMContentLoaded', () => {
             textAreaElement.value = '';
             textAreaElement.value = value;
             textAreaElement.setAttribute("style", "height: " + textAreaElement.scrollHeight + "px;");
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("You can't edit this comment"));
+            console.error(error);
         });
     };
 
@@ -224,10 +287,15 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             let updatedContentElement = stringToDom(result.data, '.js-updated-comment');
             form.parentElement.replaceWith(updatedContentElement);
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Modification didn't take effect!, please try again"));
+            console.error(error);
         });
     };
 
@@ -267,11 +335,16 @@ document.addEventListener('DOMContentLoaded', () => {
         fetch(url, {headers: headers}).then(response => {
             return response.json()
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             showModal(deleteModal);
             let modalContent = deleteModal.querySelector('.comment-modal-content');
             modalContent.innerHTML = result.data;
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Deletion cannot be performed!, please try again"));
+            console.error(error);
         });
     };
 
@@ -293,6 +366,10 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             if (isParent) {
                 document.getElementById("comments").outerHTML = result.data;
             } else {
@@ -312,8 +389,9 @@ document.addEventListener('DOMContentLoaded', () => {
             }
             hideModal(deleteModal);
             commentElement.remove();
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Unable to delete your comment!, please try again"));
+            console.error(error);
         });
     };
 
@@ -364,6 +442,10 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             if (result.error) {
                 alert(result.error);
             } else {
@@ -373,8 +455,9 @@ document.addEventListener('DOMContentLoaded', () => {
                     changeReactionCount(parentReactionEle, result.data.likes, result.data.dislikes);
                 }
             }
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Reaction couldn't be processed!, please try again"));
+            console.error(error);
         });
     };
 
@@ -392,6 +475,10 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             if (result.error) {
                 if (result.error.email_required) {
                     followModal.querySelector('form').setAttribute('data-target-btn-id', followButton.getAttribute('id'));
@@ -417,8 +504,60 @@ document.addEventListener('DOMContentLoaded', () => {
                     hideModal(followModal);
                 }
             }
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Subscription couldn't be processed!, please try again"));
+            console.error(error);
+        });
+    };
+
+    let loadBlockModal = blockBtn => {
+        blockModal.querySelector('#blockedCommentId').value = blockBtn.getAttribute('data-comment_id');
+        showModal(blockModal);
+    };
+
+    let updateBlockedUserComments = data => {
+        let commentsByBlockedUser = document.getElementsByClassName(`block-${data.blocked_user}`);
+        let pathElement = data.blocked ? '<path d="M7 11V7a5 5 0 0 1 10 0v4"/>' : '<path d="M7 11V7a5 5 0 0 1 9.9-1"/>';
+        let color = data.blocked ? '#E74C3C' : '#00BC8C';
+        for (let commentByBlockedUser of commentsByBlockedUser) {
+            commentByBlockedUser.setAttribute('stroke', color);
+            commentByBlockedUser.querySelector('path').outerHTML = pathElement;
+        }
+    };
+
+    let toggleUserBlock = form => {
+        let formDataQuery = null;
+        if (form) {
+            let formData = serializeObject(form);
+            formDataQuery = convertFormDataToURLQuery(formData);
+        }
+        let url = form.getAttribute('data-url');
+        fetch(url, {
+            method: 'POST',
+            headers: headers,
+            body: formDataQuery,
+        }).then(response => {
+            return response.json();
+        }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
+            const infoElement = document.getElementById('comments').querySelector('.js-comment');
+            let msg = '';
+            if (result.error) {
+                msg = result.error.detail;
+            } else if (result.data) {
+                let state = result.data.blocked ? 'blocked' : 'unblocked';
+                msg = gettext(`User ${result.data.blocked_user} has been successfully ${state}`);
+                updateBlockedUserComments(result.data);
+                form.querySelector('textarea').value = '';
+            }
+            createInfoElement(infoElement, 'success', msg);
+            hideModal(blockModal);
+        }).catch((error) => {
+            alert(gettext("Blocking this user couldn't be processed!, please try again"));
+            console.error(error);
         });
     };
 
@@ -509,6 +648,10 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             if (result.error) {
                 alert(result.error)
             } else {
@@ -525,8 +668,9 @@ document.addEventListener('DOMContentLoaded', () => {
                 hideModal(flagModal);
                 createInfoElement(flagButton.closest('.js-parent-comment'), result.data.status, result.msg);
             }
-        }).catch(() => {
+        }).catch((error) => {
             alert(gettext("Flagging couldn't be processed!, please try again"));
+            console.error(error);
         });
     };
 
@@ -592,6 +736,10 @@ document.addEventListener('DOMContentLoaded', () => {
         }).then(response => {
             return response.json();
         }).then(result => {
+            if (result.status === 403) {
+                alert(result.reason);
+                return;
+            }
             if (result.error) {
                 alert(result.error);
                 return;
@@ -637,11 +785,14 @@ document.addEventListener('DOMContentLoaded', () => {
     document.addEventListener('click', (event) => {
         removeTargetElement();
         if (event.target && event.target !== event.currentTarget) {
-            if (event.target === deleteModal || event.target === flagModal || event.target === followModal ||
+            if (event.target === deleteModal || event.target === flagModal || event.target === followModal || event.target === blockModal ||
+                event.target === createAnonymousCommentModal ||
                 event.target.closest('.modal-close-btn') || event.target.closest('.modal-cancel-btn')) {
                 hideModal(deleteModal);
                 hideModal(flagModal);
                 hideModal(followModal);
+                hideModal(blockModal);
+                hideCreateAnonymousCommentModal();
             } else if (event.target.closest('.js-reply-link')) {
                 event.preventDefault();
                 replyLink(event.target);
@@ -672,6 +823,9 @@ document.addEventListener('DOMContentLoaded', () => {
             } else if (event.target.closest('.js-three-dots')) {
                 event.preventDefault();
                 openThreeDostMenu(event.target.closest('.js-three-dots'));
+            } else if (event.target.closest('.js-comment-block')) {
+                event.preventDefault();
+                loadBlockModal(event.target.closest('.js-comment-block'));
             }
         }
     }, false);
@@ -693,6 +847,9 @@ document.addEventListener('DOMContentLoaded', () => {
                 event.preventDefault();
                 let followButton = document.getElementById(event.target.getAttribute('data-target-btn-id'));
                 toggleFollow(followButton, event.target);
+            } else if (event.target.classList.contains('js-comment-block-form')) {
+                event.preventDefault();
+                toggleUserBlock(event.target);
             }
         }
     }, false);
