@@ -3,16 +3,19 @@ from django.shortcuts import get_object_or_404
 from django.views import View
 
 from comment.models import Comment, Flag, FlagInstance
-from comment.mixins import CanSetFlagMixin, CanEditFlagStateMixin, DABResponseData
-from comment.responses import UTF8JsonResponse
+from comment.mixins import CanSetFlagMixin, CanUpdateFlagStateMixin, BaseCommentMixin
+from comment.responses import UTF8JsonResponse, DABResponseData
 from comment.messages import FlagInfo, FlagError
 
 
-class SetFlag(CanSetFlagMixin, View, DABResponseData):
+class SetFlag(CanSetFlagMixin, BaseCommentMixin, View, DABResponseData):
     comment = None
 
     def get_object(self):
-        self.comment = get_object_or_404(Comment, pk=self.kwargs.get('pk'))
+        self.comment = get_object_or_404(
+            Comment.objects.select_related('user', 'reaction', 'flag'),
+            pk=self.kwargs.get('pk')
+        )
         return self.comment
 
     def post(self, request, *args, **kwargs):
@@ -29,6 +32,7 @@ class SetFlag(CanSetFlagMixin, View, DABResponseData):
                 self.msg = FlagInfo.UNFLAGGED_SUCCESS
 
             self.data.update({'status': 0})
+            self.status = 200
         except ValidationError as e:
             self.error = e.message
             self.status = 400
@@ -36,17 +40,21 @@ class SetFlag(CanSetFlagMixin, View, DABResponseData):
         return UTF8JsonResponse(self.json(), status=self.status)
 
 
-class ChangeFlagState(CanEditFlagStateMixin, View, DABResponseData):
+class ChangeFlagState(CanUpdateFlagStateMixin, View, DABResponseData):
     comment = None
 
     def get_object(self):
-        self.comment = get_object_or_404(Comment, pk=self.kwargs.get('pk'))
+        self.comment = get_object_or_404(
+            Comment.objects.select_related('user', 'reaction', 'flag'),
+            pk=self.kwargs.get('pk')
+        )
         return self.comment
 
     def post(self, request, *args, **kwargs):
         state = request.POST.get('state')
         try:
             self.comment.flag.toggle_state(state, request.user)
+            self.status = 200
         except ValidationError:
             self.error = FlagError.STATE_CHANGE_ERROR
             self.status = 400
